Attribute VB_Name = "ModQstats1"
Option Explicit ' Requires programmer to declare all variables
Public FPInQ() As Double  'input flow values
Public LFPInQ() As Double 'nonzero input flow values
Public LF3PInQ() As Double 'nonzero input flow values
Public lngHeadLine As Long ' header line number for error output
Public lngDataLine As Long ' header line number for error output


Sub ChkSta(strStaNum As String, iEc As Long, intmode As Long)
'
' Purpose:
' To read and check station number from text1.txt
' History:
' Version 1.0 May 2004 by G.E. Granato
' strStaNum              ' input NWIS station number as string
' iec                    ' output error code 0 = everything ok
' intmode                ' mode 1= interactive 2= batch
' Arguments
Dim ci As Long                 ' check loop integer 1
Dim cj As Long                 ' check loop Integer
Dim cnum As Long               ' check number sum
Dim n1 As Long                 ' Integer test variable
Dim n2 As Long                 ' Integer count variable
Dim ck2 As Variant                ' variant
Dim StrErrMsg As String           ' Error string
On Error GoTo Err_ChkSta:
iEc = 0
' trim leading or trailing spaces
   strStaNum = Trim(strStaNum)
'  trim leading or trailing tab
  If Left(strStaNum, 1) = Chr(9) Then
   n2 = Len(strStaNum)
   strStaNum = Right(strStaNum, (n2 - 1))
  End If
  If Right(strStaNum, 1) = Chr(9) Then
   n2 = Len(strStaNum)
   strStaNum = Left(strStaNum, (n2 - 1))
  End If
' retrim leading or trailing spaces
   strStaNum = Trim(strStaNum)
' check for proper length ' 8-12 characters
10 n2 = Len(strStaNum)
If n2 > 7 And n2 < 13 Then
 'check each character to ensure it is a number
  cnum = 0
  For ci = 1 To n2
   n1 = Mid(strStaNum, ci, 1)
   For cj = 0 To 9
    If n1 = cj Then
     cnum = cnum + 1
     GoTo 15
    End If
   Next cj
15  Next ci
  If cnum = n2 Then
   iEc = 0
   Exit Sub
  Else
   iEc = 2 'nonnumeric response
   Exit Sub
  End If
 ' if less than 8 declare error 1 and exit
 ElseIf n2 < 8 Then
  iEc = 1 'error station number too short
  Exit Sub
 'if greater than 8 try to trim left or right
 ElseIf n2 > 12 Then
  iEc = 3 'error station number too long
 Exit Sub
 End If
Err_ChkSta:
 If Err.Number = 13 Then
  If intmode = 1 Then
   StrErrMsg = " Error character number: " & ci & " is not an integer please correct"
   MsgBox StrErrMsg
  End If
 Else
  If intmode = 1 Then
  MsgBox "Other Error"
  End If
 End If
 iEc = 2
 Exit Sub
End Sub
Sub ChkFiles(strOutDir As String, strStaNum As String, intimode As Long, intErrCode As Long)
'
' Purpose:
' To check that header file and data file for station number
' exist and have some data
' History:
' Version 1.0 May 2004 by G.E. Granato
' strStaNum           'input NWIS station number as string
' IntErrCode          'output error code 0 = everything ok
'                     ' 1= no header file 2 = no data file
'                     ' 3 = both
' intimode            ' mode 1= interactive 2= batch
' Arguments
Dim bnFileExists As Boolean     ' Check variable for file existance
Dim intErrorSum As Long      ' error code builer
Dim strHeadFile As String       ' header file name
Dim strDatFile As String        ' data file name
Dim StrErrMsg As String             ' Error Message
' Initialize vars on call
intErrorSum = 0
intErrCode = 0
'Check that header file exists and has data
   strHeadFile = strOutDir & strStaNum & "H.txt"
   bnFileExists = bnCheckFile(strHeadFile)
   If bnFileExists = False Then
    intErrorSum = intErrorSum + 1
    If intimode = 1 Then
     StrErrMsg = "Error: Header File: " & Chr(13) & Chr(10) & _
     strHeadFile & Chr(13) & Chr(10) & _
     "Does Not Exist."
     MsgBox StrErrMsg
    End If
   Else
   'check if file empty
    If FileLen(strHeadFile) = 0 Then    'vb6 intrinsic file length function
     intErrorSum = intErrorSum + 1
     If intimode = 1 Then
      StrErrMsg = "Error: Header File: " & Chr(13) & Chr(10) & _
      strHeadFile & Chr(13) & Chr(10) & _
      "Has no data."
      MsgBox StrErrMsg
     End If
    End If
   End If
  'Check that data file exists
   strDatFile = strOutDir & strStaNum & "Q.txt"
   bnFileExists = bnCheckFile(strDatFile)
   If bnFileExists = False Then
    intErrCode = 2                     'no data file
    If intimode = 1 Then
     StrErrMsg = "Error: Data File: " & Chr(13) & Chr(10) & _
     strDatFile & Chr(13) & Chr(10) & _
     "Does Not Exist."
     MsgBox StrErrMsg
    End If
   Else
   'check if file empty
    If FileLen(strDatFile) = 0 Then    'vb6 intrinsic file length function
     intErrCode = 2                     'no data file
     If intimode = 1 Then
      StrErrMsg = "Error: Data File: " & Chr(13) & Chr(10) & _
      strDatFile & Chr(13) & Chr(10) & _
      "Has no data."
      MsgBox StrErrMsg
     End If
    End If
   End If
intErrorSum = intErrorSum + intErrCode
intErrCode = intErrorSum
Exit Sub
End Sub
Sub GetHeader(Strfname As String, strSite_no As String, strStation_nm As String, _
 strDec_lat_va As String, StrDec_long_va As String, strState_cd As String, strAlt_va As String, _
 strHUC_cd As String, strDArea_va As String, strCArea_va As String, lngHL As Long, intErrCode As Long)
'
' Purpose:
' To open and obtain data from header file
'
' History:
' Version 1.0 May 2004 by G.E. Granato
' Arguments
' strfname           -- File name
' strSite_no         -- Site identification number
' strStation_nm      -- Site name
' strDec_lat_va      -- Decimal latitude
' strDec_long_va     -- Decimal longitude
' strState_cd        -- State code
' strHUC_cd          -- Hydrologic unit code
' strDArea_va        -- Drainage area area or if not available Contributing drainage
' strCArea_va        -- Contributing drainage area or if not available Drainage area
' lngHL              -- line counter to see where error occurs
' intErrCode         -- error code flag to alert user that error has occured
'
Dim varArrLine As Variant   ' Line argument array
Dim strInstring As String   ' Line input string
Dim strDummyInString As String   ' Line input string
Dim strInvars(10, 10)        ' array to read (line, variable)
Dim intUnitNum As Long   ' Read file unitnumber
Dim intcountline As Long ' counts station lines
Dim intI As Long         ' index variable
' open and read header file
On Error GoTo Err_GetHeader:
intErrCode = 0
intcountline = 0
intUnitNum = FreeFile
Open Strfname For Input As intUnitNum
10:  lngHL = lngHL + 1
     Line Input #intUnitNum, strInstring           ' Read comment lines
     If Left(strInstring, 1) = "#" Then GoTo 10:
     ' read header line 2
     lngHL = lngHL + 1
     Line Input #intUnitNum, strInstring
     ' read data line
20:  intcountline = intcountline + 1
     lngHL = lngHL + 1
     Line Input #intUnitNum, strInstring
     strDummyInString = Trim(strInstring)
     If ((intcountline = 1) And (strDummyInString = "")) Then
      intErrCode = 1
      Exit Sub
     ElseIf (strDummyInString = "") Then
      GoTo 30
     End If
     ' Parse the text line by tab char (0) is first field
     varArrLine = Split(strInstring, Chr(9))
     ' assign parsed values
       strInvars(intcountline, 1) = varArrLine(0)        ' site #
       strInvars(intcountline, 2) = varArrLine(1)        ' station name
       strInvars(intcountline, 3) = varArrLine(2)        ' latitude
       strInvars(intcountline, 4) = varArrLine(3)        ' longitude
       strInvars(intcountline, 5) = varArrLine(6)        ' state code
       strInvars(intcountline, 10) = varArrLine(7)       ' altitude
       strInvars(intcountline, 6) = varArrLine(10)       ' HUC
       strInvars(intcountline, 7) = varArrLine(11)       ' begin date
       strInvars(intcountline, 8) = varArrLine(12)       ' end date
       strInvars(intcountline, 9) = Trim(varArrLine(15)) ' discharge count
     ' Take next value
     If EOF(intUnitNum) = False Then GoTo 20:
     ' close file
30      Close intUnitNum
     ' Decide which value to keep
     ' If more than one station alert user
       If (intcountline > 2) Then intErrCode = 1
       strSite_no = strInvars(1, 1)
       If intcountline = 1 Then intcountline = 2
       For intI = 1 To (intcountline - 1)
        If (strInvars(intI, 9) <> "0") Or (strInvars(intI, 9) <> "") Then
         strStation_nm = strInvars(intI, 2)
         strDec_lat_va = strInvars(intI, 3)
         StrDec_long_va = strInvars(intI, 4)
         strState_cd = strInvars(intI, 5)
         strAlt_va = strInvars(intI, 10)
         strHUC_cd = strInvars(intI, 6)
         strDArea_va = strInvars(intI, 7)
         strCArea_va = strInvars(intI, 8)
         Exit Sub
        End If
        Next intI
        Exit Sub
Err_GetHeader:
 ' something went wrong
 intErrCode = -1
 Exit Sub
 End Sub

Sub GetQ(Strfname As String, StrSN As String, FPInQ, LFPInQ, bdate As Date, edate As Date, _
lngQCount As Long, lngSkips As Long, lngZedQ As Long, lngDL As Long, intErrCode As Long, lngMyE As Long)
'
' Purpose:
' To open and obtain data from flow file
'
' History:
' Version 1.0 May 2004 by G.E. Granato ggranato@usgs.gov
' Arguments
' strfname           -- File name
' strSN              -- Site identification number
' FPInQ              -- Flow array
' LFPInQ             -- Flow array for log analysis
' bdate              -- Begin Date
' edate              -- End Date
' lngQcount          -- Number of recorded flows
' lngNZcount         -- number of nonzero flows
' lngSkips           -- Number of skips in the record
' lngZedQ            -- Number of zero flows
' lngDL              -- Line-number counter to record where error occurs
' IntErrCode          'output error code 0 = ok 1 = warning 2 = error
' Arguments
Dim varArrLine As Variant      ' Line argument array
Dim strInstring As String      ' Line input string
Dim strDummyInString As String ' Line input string
Dim intUnitNum As Long      ' Read file unitnumber
Dim intgap As Long          ' gap in days
Dim lngcountline1 As Long      ' counts all lines in file
Dim lngcountline2 As Long      ' counts header lines in file
Dim lngNumQ As Long            ' total lines of flow in file
Dim lngi As Long               ' index variable
Dim lngk As Long            ' index variable
Dim lngk2 As Long           ' index variable
Dim yesterday As Date          ' date variable
Dim today As Date              ' date variable
On Error GoTo Err_GetQ:
' Initialize counters and error variables
intErrCode = 0
lngcountline1 = 0
lngcountline2 = 0
lngNumQ = 0
lngQCount = 0
lngSkips = 0
lngZedQ = 0
lngk = 0
lngk2 = 0
lngMyE = 0
intUnitNum = FreeFile
' open and read Data file to obtain the number of lines in file
Open Strfname For Input As intUnitNum
 Do While EOF(intUnitNum) = False
  lngcountline1 = lngcountline1 + 1
  Line Input #intUnitNum, strInstring           ' Read comment lines
 Loop
Close intUnitNum
' open and read Data file to obtain data
' calculate the number of header lines in file
Open Strfname For Input As intUnitNum
10:  lngDL = lngDL + 1
     Line Input #intUnitNum, strInstring           ' Read comment lines
     If Left(strInstring, 1) = "#" Then
      lngcountline2 = lngcountline2 + 1
      GoTo 10:
     End If
      lngcountline2 = lngcountline2 + 1
     ' read header line 2
     lngDL = lngDL + 1
     Line Input #intUnitNum, strInstring
     lngcountline2 = lngcountline2 + 1
     lngNumQ = lngcountline1 - lngcountline1
     ' read data lines
     lngk = 0
     ' Redimension flow array to the number of lines in the file
     lngNumQ = lngcountline1 - lngcountline2
       If lngNumQ = 0 Then
        intErrCode = 2
        Close intUnitNum
        Exit Sub
       End If
     ReDim FPInQ(lngNumQ) As Double
     ReDim LFPInQ(lngNumQ) As Double
     ' Read Data
20:  If EOF(intUnitNum) = True Then GoTo 30
     lngDL = lngDL + 1
     Line Input #intUnitNum, strInstring
     ' on blank line exit
       If lngk = 0 And strInstring = "" Then
        intErrCode = 2
        Close intUnitNum
        Exit Sub
       ElseIf strInstring = "" Then
        Close intUnitNum
        Exit Sub
       End If
     ' Parse the text line by tab char (0) is first field
     varArrLine = Split(strInstring, Chr(9))
     ' if flow value is null dont count line
        If (varArrLine(3) = "") Then GoTo 20
     ' advance input variable
       lngk = lngk + 1
     ' If station number does not equal expected station number set error code and exit
       strDummyInString = varArrLine(1)
       If strDummyInString <> StrSN Then
        intErrCode = 6
        Close intUnitNum
        Exit Sub
       End If
     ' Count e's
       If varArrLine(4) = "e" Then lngMyE = lngMyE + 1
     ' get beginning date
      today = varArrLine(2)
      If lngk = 1 Then
       bdate = today
       yesterday = today
      Else
       intgap = today - yesterday
       If intgap < 1 Then
        intErrCode = 1
        Close intUnitNum
        Exit Sub
       ElseIf intgap > 1 Then
        lngSkips = lngSkips + 1
       End If
      End If
     ' update flow counter
        lngQCount = lngQCount + 1
     ' set end date to current date
        edate = today
     ' record flow value
       FPInQ(lngk) = varArrLine(3)
     ' check and record zero flows
       If FPInQ(lngk) <= 0 Then
        lngZedQ = lngZedQ + 1
       Else
        lngk2 = lngk2 + 1
        LFPInQ(lngk2) = FPInQ(lngk)
       End If
     ' do next flow
       yesterday = today
       GoTo 20:
30     Close intUnitNum
       Exit Sub
Err_GetQ:
 intErrCode = 2
 Exit Sub
End Sub
Sub STATS01(RinQ() As Double, num As Long, avg As Double, var As Double, sdev As Double, skew As Double)
'
' Purpose:
' Takes array RinQ and returns statistics
' utilizes equations in:
' U.S. Interagency Advisory Commitee on Water Data, 1982,
' Guidelines for determining flood flow frequency, Bulletin 17-B
' of the Hydrology Subcommittee: Reston Virginia,
' U.S. Geological Survey Office of Water Data Coordination, 183 p.
'
' History:
' Version 1.0 May 2004 by G.E. Granato ggranato@usgs.gov
'
' Subroutine arguments
'
' num    number of values in array
' avg    average of array
' var    variance of array
' sdev   standard deviation of array
' skew   skew coefficient of array
'
' Internal variables
Dim ssum As Double        ' sum of input values or higher moment sum
Dim sVARsum As Double     ' sum for higher moments
Dim sSKWsum As Double     ' product for higher moments
Dim scnt As Double        ' single count
Dim lngn As Long          ' Max counter for warm-up loop
Dim lngi As Long          ' First for-next index variable
Dim lngk As Long          ' quotient from integer division
ssum = 0#
scnt = num / 1#
' Do first pass to calculate average
For lngi = 1 To num
ssum = ssum + RinQ(lngi)
Next lngi
' Calculate average
avg = ssum / scnt
' initialize for higher moments
var = 0#
sdev = 0#
skew = 0#
ssum = 0#
sVARsum = 0#
sSKWsum = 0#
' second pass to get higher moments
For lngi = 1 To num
 ssum = (RinQ(lngi) - avg)
 sVARsum = sVARsum + (ssum * ssum)
 sSKWsum = sSKWsum + (ssum * ssum * ssum)
Next lngi
' calculate final values
var = sVARsum / (scnt - 1#)
sdev = Sqr(var)
' if var=0 division by zero error
If (var <> 0#) Then
 skew = (scnt * sSKWsum) / ((scnt - 1#) * (scnt - 2#) * (sdev * sdev * sdev))
Else
 skew = -0.000000000191
End If
End Sub
Sub DoLMoments(sRinQ() As Double, lnum As Long, sb0 As Double, sb1 As Double, sb2 As Double, sb3 As Double, _
slam1 As Double, slam2 As Double, slam3 As Double, slam4 As Double, slcv As Double, slskew As Double, slKurt As Double)
'
' Purpose:
' Takes array sRinQ and returns l-moment statistics
' utilizes equations in:
' Stedinger, J.R., Vogel, R.M., Foufoula-Georgiou, Efi, 1993
' Chapter 18--Frequency analysis of extreme events: in
' Maidment, D.R., (ed.), Handbook of Hydrology:
' New York, McGraw-Hill, p. 18.1-18.66.
'
' History:
' Version 1.0 May 2007 by G.E. Granato ggranato@usgs.gov
'
' Subroutine arguments
'
' lnum    number of values in array
' sb0    average of array calculated in sb0
' sb1    1st Probability weighted moment
' sb2    2nd Probability weighted moment
' sb3    3rd Probability weighted moment
' slam1  1st l-moment =sbo
' slam2  2nd l-moment =2*sb1-sb0
' slam3  3rd l-moment =6*sb2-6*sb1+sb0
' slam4  4th l-moment =20*sb3-30*sb2+12*sb1-sb0
' slcv   l-moment coefficient of variation slam2/slam1
' slskew   l-moment coefficient skewness slam3/slam2
' slKurt   l-moment coefficient skewness slam4/slam2
'
' Internal variables
Dim lngj As Long
Dim dJ As Double
Dim dNo As Double
Dim dN1 As Double
Dim dN2 As Double
Dim dN3 As Double
Dim db1 As Double
Dim db2 As Double
Dim db3 As Double


' On Error GoTo lmom_err:
' initialize
 db1 = 0#
 db2 = 0#
 db3 = 0#
 dNo = CDbl(lnum)
 dN1 = dNo - 1#
 dN2 = dNo - 2#
 dN3 = dNo - 3#
'***********************************
' Calculate probability weighted moments
'***********************************
For lngj = 1 To lnum
 dJ = CDbl(lngj)
 db1 = db1 + (((dJ - 1#) * sRinQ(lngj)) / (dNo * dN1))
 db2 = db2 + (((dJ - 1#) * (dJ - 2#) * sRinQ(lngj)) / (dNo * dN1 * dN2))
 db3 = db3 + (((dJ - 1#) * (dJ - 2#) * (dJ - 3#) * sRinQ(lngj)) / (dNo * dN1 * dN2 * dN3))
Next lngj
 
'***********************************
' calculate l-moments
'***********************************
 slam1 = sb0
 slam2 = (2# * db1) - sb0
 slam3 = (6# * db2) - (6# * db1) + sb0
 slam4 = (20# * db3) - (30# * db2) + (12# * db1) - sb0
 sb1 = CSng(db1)
 sb2 = CSng(db2)
 sb3 = CSng(db3)
'***********************************
' calculate l-moment ratios
 slcv = slam2 / slam1
 slskew = slam3 / slam2
 slKurt = slam4 / slam2
'***********************************
Exit Sub
'***********************************
lmom_err:
 slam1 = -9999#
 slam2 = -9999#
 slam3 = -9999#
 slam4 = -9999#
 sb1 = -9999#
 sb2 = -9999#
 sb3 = -9999#
 slcv = -9999#
 slskew = -9999#
 slKurt = -9999#
Exit Sub

End Sub

Sub ssort(Arr() As Double, lngcnt As Long)
'
' Purpose:
' Takes array Arr and sorts
' lngcnt  number of values in array
'
' Arguments
Dim stmp As Double
Dim intj As Long
Dim intk As Long
Dim intL As Long
Dim intInc As Long
Dim lngc As Long
   lngc = lngcnt
   intInc = 1
10 intInc = 3 * intInc + 1
   If (intInc <= lngc) Then GoTo 10
20 intInc = intInc \ 3
   For intj = intInc + 1 To lngc
    stmp = Arr(intj)
    intk = intj
30  intL = intk - intInc
    If (Arr(intL) >= stmp) Then
     Arr(intk) = Arr(intL)
     intk = intk - intInc
     If (intk <= intInc) Then GoTo 40
     GoTo 30
    End If
40    Arr(intk) = stmp
   Next intj
   If (intInc > 1) Then GoTo 20
End Sub
Sub PutFileHeader(StrOutFile As String)
'
' Purpose:
' To take filename and create header information
' StrOutFile is the output filename
' History:
' Version 1.0 May 2004 by G.E. Granato
' Arguments
Dim intfnum As Long       ' File unit number for error file
Dim strHeadstr As String     ' String to create header lines
Dim dtoday As Date            ' Date for today
Dim tb As String             ' tab character
tb = Chr(9)
' Get current date and time
 dtoday = Now
' open file
  intfnum = FreeFile
  Open StrOutFile For Append As intfnum
' Get lines and print them
  Print #intfnum, "#"
  Print #intfnum, "#"
  Print #intfnum, "# NWIS Streamflow Statistics (QSTATS--version 1.0)"
  Print #intfnum, "# Data from U.S. Geological Survey "
  Print #intfnum, "# National Water information System (NWIS)"
  Print #intfnum, "# http://waterdata.usgs.gov/nwis/"
  Print #intfnum, "#"
  Print #intfnum, "# Processed on " & Format(dtoday, "YYYY-MM-DD HH:MM:SS")
  Print #intfnum, "#"
  Print #intfnum, "#  site_no         -- Site identification number"
  Print #intfnum, "#  station_nm      -- Site name"
  Print #intfnum, "#  dec_lat_va      -- Decimal latitude"
  Print #intfnum, "#  dec_long_va     -- Decimal longitude"
  Print #intfnum, "#  state_cd        -- State code"
  Print #intfnum, "#  huc_cd          -- Hydrologic unit code"
  Print #intfnum, "#  drain_area_va   -- Drainage area"
  Print #intfnum, "#  contrib_drain_area_va  -- Contributing drainage area"
  Print #intfnum, "#  discharge_begin_date   -- Daily streamflow data begin date"
  Print #intfnum, "#  discharge_end_date     -- Daily streamflow data end date"
  Print #intfnum, "#  discharge_count_nu     -- Daily streamflow data count"
  Print #intfnum, "#  discharge_skipcount_nu -- Daily streamflow number of skips in data count"
  Print #intfnum, "#  discharge_zerocount_nu -- Daily streamflow number of zero flows in data"
  Print #intfnum, "#  discharge_zeroprop_va  -- Daily streamflow proportion of zero flows in data"
  Print #intfnum, "#  discharge_AMean_va     -- Arithmetic average of daily streamflows, in cubic-feet per-second"
  Print #intfnum, "#  discharge_ASDEV_va     -- Arithmetic standard deviation of daily streamflows, in cubic-feet per-second"
  Print #intfnum, "#  discharge_ASkew_va     -- Arithmetic skew of daily streamflows, unitless"
  Print #intfnum, "#  discharge_AMed_va      -- Arithmetic median of daily streamflows, in cubic-feet per-second"
  Print #intfnum, "#  discharge_LMean_va     -- Retransformed average of Log10(nonzero daily-streamflows), in cubic-feet per-second"
  Print #intfnum, "#  discharge_LSDEV_va     -- Retransformed standard deviation of Log10(nonzero daily-streamflows), unitless"
  Print #intfnum, "#  discharge_LSkew_va     -- Skew of Log10(nonzero daily-streamflows), unitless"
  'Print #intfnum, "#  discharge_LCOV_va      -- Coefficient of variation of Log10(nonzero daily-streamflows), unitless"
  strHeadstr = "site_no" & tb & "station_nm" & tb & "dec_lat_va" & tb & "dec_long_va" & tb & "state_cd" & tb & _
  "huc_cd" & tb & "drain_area_va" & tb & "contrib_drain_area_va" & tb & "discharge_begin_date" & tb & _
  "discharge_end_date" & tb & "discharge_count_nu" & tb & "discharge_skipcount_nu" & tb & _
  "discharge_zerocount_nu" & tb & "discharge_zeroprop_va" & tb & "discharge_AMean_va" & tb & "discharge_ASDEV_va" & tb & "discharge_ASkew_va" & tb & _
  "discharge_AMed_va" & tb & "discharge_LMean_va" & tb & "discharge_LSDEV_va" & tb & "discharge_LSkew_va"
  Print #intfnum, strHeadstr
'  strHeadstr = "15s" & tb & "50s" & tb & "16n" & tb & "16n" & tb & "2s" & tb & "8s" & tb & "8n" & tb & _
'  "8n" & tb & "10s" & tb & "10s" & tb & "8n" & tb & "8n" & tb & "8n" & tb & "10n" & tb & "10n" & tb & "10n" & tb & _
'  "10n" & tb & "10n" & tb & "10n" & tb & "10n" & tb & "10n" & tb & "10n"
'  Print #intfnum, strHeadstr
  Close intfnum
End Sub
Sub PutGISHeader(StrOutFile As String)
'
' Purpose:
' To take filename and create header information for the GIS output file
' StrOutFile is the output filename
' History:
' Version 1.0 May 2004 by G.E. Granato
' Arguments
Dim intfnum As Long       ' File unit number for error file
Dim strHeadstr As String     ' String to create header lines
Dim dtoday As Date            ' Date for today
Dim tb As String             ' tab character
tb = Chr(9)
' Get current date and time
 dtoday = Now
' open file
  intfnum = FreeFile
  Open StrOutFile For Append As intfnum
' Get lines and print them
  Print #intfnum, "#"
  Print #intfnum, "#"
  Print #intfnum, "# NWIS Streamflow Statistics (QSTATS--version 1.0)"
  Print #intfnum, "# Data from U.S. Geological Survey "
  Print #intfnum, "# National Water information System (NWIS)"
  Print #intfnum, "# http://waterdata.usgs.gov/nwis/"
  Print #intfnum, "#"
  Print #intfnum, "# Streamflow per unit area formated for input into a "
  Print #intfnum, "# Geographic Information System (GIS) file once this header information is stripped"
  Print #intfnum, "# Processed on " & Format(dtoday, "YYYY-MM-DD HH:MM:SS")
  Print #intfnum, "#"
  Print #intfnum, "#  dec_long_va           -- Decimal longitude"
  Print #intfnum, "#  dec_lat_va            -- Decimal latitude"
  Print #intfnum, "#  site_no               -- Site identification number"
  Print #intfnum, "#  station_nm            -- Site name"
  Print #intfnum, "#  alt_va                -- Altitude of Gage/land surface"
  Print #intfnum, "#  drain_area_va         -- Drainage area"
  Print #intfnum, "#  CSFM_drain_area_va    -- Drainage area for CFSM calculations"
  Print #intfnum, "#  discharge_count_nu    -- Daily streamflow data count"
  Print #intfnum, "#  discharge_zeroprop_va -- Daily streamflow proportion of zero flows in data"
  Print #intfnum, "#  discharge_LMean_va    -- Retransformed Mean of Log10(nonzero daily-streamflows), in cubic-feet per-second per square mile (CFSM)"
  Print #intfnum, "#  discharge_LSDEV_va    -- Retransformed standard deviation of Log10(nonzero daily-streamflows), unitless"
  ' Print #intfnum, "#  discharge_LCOV_va     -- Coefficient of variation of Log10(nonzero daily-streamflows), unitless"
  Print #intfnum, "#  discharge_LSkew_va    -- Skew of Log10(nonzero daily-streamflows), unitless"
  Print #intfnum, "#  discharge_LMed_va     -- Retransformed median of Log10(nonzero daily-streamflows), in cubic-feet per-second per square mile"
  Print #intfnum, "#  discharge_AMean_va    -- Arithmetic Mean of daily streamflows, in cubic-feet per-second per square mile (CFSM)"
  Print #intfnum, "#  discharge_ASDEV_va    -- Arithmetic standard deviation of daily streamflows, in cubic-feet per-second per square mile"
  Print #intfnum, "#  discharge_ASkew_va    -- Arithmetic skew of daily streamflows, unitless"
  Print #intfnum, "#  discharge_AMed_va     -- Arithmetic median of daily streamflows, in cubic-feet per-second per square mile"
  Print #intfnum, "#"
  strHeadstr = "dec_long_va" & tb & "dec_lat_va" & tb & "site_no" & tb & "station_nm" & tb & _
   "alt_va" & tb & "drain_area_va" & tb & "CSFM_drain_area_va" & tb & "discharge_count_nu" & tb & _
   "discharge_zeroprop_va" & tb & "discharge_LMean_va" & tb & "discharge_LSDEV_va" & tb & _
   "discharge_LSkew_va" & tb & "discharge_LMed_va" & tb & _
   "discharge_AMean_va" & tb & "discharge_ASDEV_va" & tb & "discharge_ASkew_va" & tb & "discharge_AMed_va"
   
  Print #intfnum, strHeadstr
  Close intfnum
End Sub
Sub PutLMomHeader(StrOutFile As String)
'
' Purpose:
' To take filename and create header information for the L-Moments output file
' StrOutFile is the output filename
' History:
' Version 1.0 May 2004 by G.E. Granato
' Arguments
Dim intfnum As Long       ' File unit number for error file
Dim strHeadstr As String     ' String to create header lines
Dim dtoday As Date            ' Date for today
Dim tb As String             ' tab character
tb = Chr(9)
' Get current date and time
 dtoday = Now
' open file
  intfnum = FreeFile
  Open StrOutFile For Append As intfnum
' Get lines and print them
' Get lines and print them
  Print #intfnum, "#"
  Print #intfnum, "#"
  Print #intfnum, "# NWIS Streamflow Statistics (QSTATS--version 1.0)"
  Print #intfnum, "# Data from U.S. Geological Survey "
  Print #intfnum, "# National Water information System (NWIS)"
  Print #intfnum, "# http://waterdata.usgs.gov/nwis/"
  Print #intfnum, "#"
  Print #intfnum, "# L-Moment Streamflow Statistics  "
  Print #intfnum, "# Processed on " & Format(dtoday, "YYYY-MM-DD HH:MM:SS")
  Print #intfnum, "# From equations in: "
  Print #intfnum, "# Hosking, J.R.M., and Wallis, J.R., 1997, Regional Frequency Analysis: "
  Print #intfnum, "# New York, Cambridge University Press, 224 p."
  Print #intfnum, "#"
  Print #intfnum, "#  site_no     -- Site identification number"
  Print #intfnum, "#  station_nm  -- Site name"
  Print #intfnum, "#  Q_count_nu  -- Daily streamflow data count"
  Print #intfnum, "#  Q_AMean_va  -- Arithmetic average of daily streamflows, in cubic-feet per-second (0th moment)"
  Print #intfnum, "#  Q_APWM_b1   -- Arithmetic probability weighted moment, in cubic-feet per-second (1st moment)"
  Print #intfnum, "#  Q_APWM_b2   -- Arithmetic probability weighted moment, in cubic-feet per-second (2nd moment)"
  Print #intfnum, "#  Q_APWM_b3   -- Arithmetic probability weighted moment, in cubic-feet per-second (3rd moment)"
  Print #intfnum, "#  Q_ALM_L1    -- Arithmetic 1st L-moment, in cubic-feet per-second = Q_AMean_va"
  Print #intfnum, "#  Q_ALM_L2    -- Arithmetic 2nd L-moment, in cubic-feet per-second"
  Print #intfnum, "#  Q_ALM_L3    -- Arithmetic 3rd L-moment, in cubic-feet per-second"
  Print #intfnum, "#  Q_ALM_L4    -- Arithmetic 4th L-moment, in cubic-feet per-second"
  Print #intfnum, "#  Q_ALM_CV    -- Arithmetic L-moment coefficient of variation (Q_ALM_L2/Q_ALM_L1)"
  Print #intfnum, "#  Q_ALM_Skew  -- Arithmetic L-moment coefficient of skewness (Q_ALM_L3/Q_ALM_L2)"
  Print #intfnum, "#  Q_ALM_Kurt  -- Arithmetic L-moment coefficient of kurtosis (Q_ALM_L4/Q_ALM_L2)"
  Print #intfnum, "#  "
   strHeadstr = "site_no" & tb & "station_nm" & tb & "Q_count_nu" & tb & "Q_AMean_va" & tb & "Q_APWM_b1" & tb & "Q_APWM_b2" & _
   tb & "Q_APWM_b3" & tb & "Q_ALM_L1" & tb & "Q_ALM_L2" & tb & "Q_ALM_L3" & tb & "Q_ALM_L4" & tb & "Q_ALM_CV" & tb & _
      "Q_ALM_Skew" & tb & "Q_ALM_Kurt"
   Print #intfnum, strHeadstr
  Close (intfnum)





End Sub

Sub PutLMomLogHeader(StrOutFile As String)
'
' Purpose:
' To take filename and create header information for the L-Moments LOG10 output file
' StrOutFile is the output filename
' History:
' Version 1.0 May 2004 by G.E. Granato
' Arguments
Dim intfnum As Long       ' File unit number for error file
Dim strHeadstr As String     ' String to create header lines
Dim dtoday As Date            ' Date for today
Dim tb As String             ' tab character
tb = Chr(9)
' Get current date and time
 dtoday = Now
' open file
  intfnum = FreeFile
  Open StrOutFile For Append As intfnum
' Get lines and print them
' Get lines and print them
  Print #intfnum, "#"
  Print #intfnum, "#"
  Print #intfnum, "# NWIS Streamflow Statistics (QSTATS--version 1.0)"
  Print #intfnum, "# Data from U.S. Geological Survey "
  Print #intfnum, "# National Water information System (NWIS)"
  Print #intfnum, "# http://waterdata.usgs.gov/nwis/"
  Print #intfnum, "#"
  Print #intfnum, "# L-Moment Streamflow Statistics for the common log of nonzero values"
  Print #intfnum, "# Processed on " & Format(dtoday, "YYYY-MM-DD HH:MM:SS")
  Print #intfnum, "# From equations in: "
  Print #intfnum, "# Hosking, J.R.M., and Wallis, J.R., 1997, Regional Frequency Analysis: "
  Print #intfnum, "# New York, Cambridge University Press, 224 p."
  Print #intfnum, "#"
  Print #intfnum, "#  site_no     -- Site identification number"
  Print #intfnum, "#  station_nm  -- Site name"
  Print #intfnum, "#  Q_count_nu  -- Nonzero daily streamflow data count"
  Print #intfnum, "#  Q_LMean_va  -- Common (base10) average of nonzero daily streamflows (0th moment)"
  Print #intfnum, "#  Q_LPWM_b1   -- Common (base10) probability weighted moment (1st moment)"
  Print #intfnum, "#  Q_LPWM_b2   -- Common (base10) probability weighted moment (2nd moment)"
  Print #intfnum, "#  Q_LPWM_b3   -- Common (base10) probability weighted moment (3rd moment)"
  Print #intfnum, "#  Q_LLM_L1    -- Common (base10) 1st L-moment = Q_AMean_va"
  Print #intfnum, "#  Q_LLM_L2    -- Common (base10) 2nd L-moment "
  Print #intfnum, "#  Q_LLM_L3    -- Common (base10) 3rd L-moment "
  Print #intfnum, "#  Q_LLM_L4    -- Common (base10) 4th L-moment "
  Print #intfnum, "#  Q_LLM_CV    -- Common (base10) L-moment coefficient of variation (Q_LLM_L2/Q_LLM_L1)"
  Print #intfnum, "#  Q_LLM_Skew  -- Common (base10) L-moment coefficient of skewness (Q_LLM_L3/Q_LLM_L2)"
  Print #intfnum, "#  Q_LLM_Kurt  -- Common (base10) L-moment coefficient of kurtosis (Q_LLM_L4/Q_LLM_L2)"
  Print #intfnum, "#  "
   strHeadstr = "site_no" & tb & "station_nm" & tb & "Q_count_nu" & tb & "Q_LMean_va" & tb & "Q_LPWM_b1" & tb & "Q_LPWM_b2" & _
   tb & "Q_LPWM_b3" & tb & "Q_LLM_L1" & tb & "Q_LLM_L2" & tb & "Q_LLM_L3" & tb & "Q_LLM_L4" & tb & "Q_LLM_CV" & tb & _
      "Q_LLM_Skew" & tb & "Q_LLM_Kurt"
   Print #intfnum, strHeadstr
  Close (intfnum)

End Sub
Sub Put3PLogHeader(StrOutFile As String)
'
' Purpose:
' To take filename and create header information for the 3-parameter lognormal output file
' StrOutFile is the output filename
' History:
' Version 1.0 May 2004 by G.E. Granato
' Arguments
Dim intfnum As Long       ' File unit number for error file
Dim strHeadstr As String     ' String to create header lines
Dim dtoday As Date            ' Date for today
Dim tb As String             ' tab character
tb = Chr(9)
' Get current date and time
 dtoday = Now
' open file
  intfnum = FreeFile
  Open StrOutFile For Append As intfnum
' Get lines and print them
' Get lines and print them
  Print #intfnum, "#"
  Print #intfnum, "#"
  Print #intfnum, "# NWIS Streamflow Statistics (QSTATS--version 1.0)"
  Print #intfnum, "# Data from U.S. Geological Survey "
  Print #intfnum, "# National Water information System (NWIS)"
  Print #intfnum, "# http://waterdata.usgs.gov/nwis/"
  Print #intfnum, "#"
  Print #intfnum, "# 3-Parameter Lognormal Streamflow Statistics  "
  Print #intfnum, "# Processed on " & Format(dtoday, "YYYY-MM-DD HH:MM:SS")
  Print #intfnum, "# From equations in: "
  Print #intfnum, "# Hosking, J.R.M., and Wallis, J.R., 1997, Regional Frequency Analysis: "
  Print #intfnum, "# New York, Cambridge University Press, 224 p."
  Print #intfnum, "#"
  Print #intfnum, "#  site_no     -- Site identification number"
  Print #intfnum, "#  station_nm  -- Site name"
  Print #intfnum, "#  LQ_count_nu  -- Nonzero Daily streamflow data count"
  Print #intfnum, "#  discharge_zerocount_nu -- Daily streamflow number of zero flows in data"
  Print #intfnum, "#  discharge_3PLogQLBE      -- Quantile-lower-bound estimator, in cubic-feet per-second (CFS)"
  Print #intfnum, "#  discharge_3PLogMean_va   -- Retransformed Mean of Log10(nonzero daily-streamflows), in cubic-feet per-second (CFS)"
  Print #intfnum, "#  discharge_3PLogSDEV_va   -- Retransformed standard deviation of Log10(nonzero daily-streamflows), unitless"
  Print #intfnum, "#  discharge_3PLogSkew_va   -- Skew of Log10(nonzero daily-streamflows), unitless"
  Print #intfnum, "#  "
   strHeadstr = "site_no" & tb & "station_nm" & tb & "LQ_count_nu" & tb & "discharge_zerocount_nu" & tb & _
   "discharge_3PLogQLBE" & tb & "discharge_3PLogMean_va" & tb & "discharge_3PLogSDEV_va" & tb & "discharge_3PLogSkew_va"
   Print #intfnum, strHeadstr
  Close (intfnum)


End Sub


Sub ChkNumber(strInStr As String, intEC As Long)
'
' Purpose:
' To check that a number is a single by assigning it an error condition will indicate
' text
' History:
' Version 1.0 May 2004 by G.E. Granato
'
' strInStr  Input string
' error condition
' Arguments
  Dim sTest As Double  'test input string
  ' if Error type mismatch (input is not a number) assign intEC as 1
    On Error GoTo Err_NotANumber:
  ' initialize error code
    intEC = 0
  ' test for null and for conversion
    If strInStr = "" Then
     intEC = 2
    Else
     sTest = strInStr
    End If
  Exit Sub
Err_NotANumber:
  intEC = 1
  Exit Sub
End Sub
Sub PosString(strInstring As String, strlookstr As String, intcount As Long)
'
' Purpose:
' To get the location of an input string
' History:
' Version 1.0 July 2004 by G.E. Granato
' strStaFile is a string that includes the dive, directory and filename
' for the file of interest
'
Dim strSearchMe As String
Dim strFindMe As String
strSearchMe = UCase(strInstring)
strFindMe = UCase(strlookstr)
intcount = InStr(strSearchMe, strFindMe)
End Sub


Public Function bnCheckFile(strStaFile As String) As Boolean
'
' Purpose:
' To check that file exists for processing or other action
' History:
' Version 1.0 May 2004 by G.E. Granato
' strStaFile is a string that includes the dive, directory and filename
' for the file of interest
'
' Arguments
  If Dir$(strStaFile) <> "" Then  'vb intrinsic file function
   bnCheckFile = True
  Else
   bnCheckFile = False
  End If
End Function

Public Function GEGMED(indata() As Double, lngcont As Long) As Double
'
' Purpose:
' To take sorted array and array size find median
' History:
' Version 1.0 January 2004 by G.E. Granato
' indata is a single precision array of input values
' lngcount is a long value indicating the size of the array
'
' Arguments
Dim lngfi As Long
Dim lngfj As Long
Dim lngfk As Long
Dim lngfin As Long
lngfj = lngcont Mod 2 'modulus provides remainder
lngfk = lngcont \ 2 'integer division truncates
' get median value
If (lngfj = 0) Then  'this is an even number
 GEGMED = (indata(lngfk) + indata(lngfk + 1)) / 2#
Else 'this is an odd number
 GEGMED = indata(lngfk + 1)
End If
End Function

Public Function QuickSortArray(varArray As Variant, _
                        Optional lngFirst As Long = 0, _
                        Optional lngLast As Long = 0) As Variant
    '
    ' This function is royalty free open source code
    ' Provided with the Microsoft Office Developer Code Librarian
    ' for use and distribution according to the Code Librarian EULA
    ' QuickSort algorithm used to sort the items
    ' in the varArray array.
    
    Dim lngLow      As Long
    Dim lngHigh     As Long
    Dim lngMiddle   As Long
    Dim varTempVal  As Variant
    Dim varTestVal  As Variant
    
    If lngFirst = -1 Then lngFirst = LBound(varArray)
    If lngLast = -1 Then lngLast = UBound(varArray)
        
    If lngFirst < lngLast Then
        lngMiddle = (lngFirst + lngLast) / 2
        varTestVal = varArray(lngMiddle)
        lngLow = lngFirst
        lngHigh = lngLast
        Do
            Do While varArray(lngLow) < varTestVal
                lngLow = lngLow + 1
            Loop
            Do While varArray(lngHigh) > varTestVal
                lngHigh = lngHigh - 1
            Loop
            If (lngLow <= lngHigh) Then
                varTempVal = varArray(lngLow)
                varArray(lngLow) = varArray(lngHigh)
                varArray(lngHigh) = varTempVal
                lngLow = lngLow + 1
                lngHigh = lngHigh - 1
            End If
        Loop While (lngLow <= lngHigh)
        If lngFirst < lngHigh Then QuickSortArray varArray, lngFirst, lngHigh
        If lngLow < lngLast Then QuickSortArray varArray, lngLow, lngLast
    End If
End Function

